package com.wmdigit.transfer.converter;


import com.wmdigit.transfer.domain.DomainBean;

import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public interface BeanConverter<T extends DomainBean> {
    public static final String DEFAULT_ENCODING = "GBK";
    public static final String UNICODE_ENCODING = "Unicode";

    public final  class CC {
        public static int calcCrc16(byte[] bArr, int i, int i2) {
            int i3 = 0;
            while (i < i2) {
                i3 ^= bArr[i] << 8;
                for (int i4 = 0; i4 < 8; i4++) {
                    i3 = (32768 & i3) != 0 ? (i3 << 1) ^ 4129 : i3 << 1;
                }
                i++;
            }
            return i3;
        }

        public static boolean hasLength(String str) {
            return str != null && !str.isEmpty();
        }

        public static String trimCharacter(String str, char c) {
            if (!hasLength(str)) {
                return str;
            }
            StringBuilder sb = new StringBuilder(str);
            while (sb.length() > 0 && sb.charAt(0) == c) {
                sb.deleteCharAt(0);
            }
            while (sb.length() > 0 && sb.charAt(sb.length() - 1) == c) {
                sb.deleteCharAt(sb.length() - 1);
            }
            return sb.toString().trim();
        }

        public static String bytesToText(ByteBuffer byteBuffer, int i) throws UnsupportedEncodingException {
            byte[] bArr = new byte[i];
            byteBuffer.get(bArr);
            return trimCharacter(new String(bArr, BeanConverter.DEFAULT_ENCODING).replace("\u0000", ""), (char) 0);
        }

        public static String doubleBytesToText(ByteBuffer byteBuffer, int i) throws UnsupportedEncodingException {
            ByteBuffer order = ByteBuffer.allocate(i * 2).order(ByteOrder.BIG_ENDIAN);
            for (int i2 = 0; i2 < i; i2++) {
                order.putShort(byteBuffer.getShort());
            }
            return trimCharacter(new String(order.array(), BeanConverter.UNICODE_ENCODING).replace("\u0000", ""), (char) 0);
        }
    }

    ByteBuffer fromBean(T t);

    T toBean(ByteBuffer byteBuffer) throws UnsupportedEncodingException;

    T toDeleteBean(ByteBuffer byteBuffer);
}
