package com.wmdigit.transfer.wrapper;

import android.util.Log;

import com.wmdigit.transfer.constant.ControlResponseEnum;
import com.wmdigit.transfer.constant.FixedWeightUsageEnum;
import com.wmdigit.transfer.constant.ScaleTypeEnum;
import com.wmdigit.transfer.converter.BeanConverter;
import com.wmdigit.transfer.converter.CommandHeaderConverter;
import com.wmdigit.transfer.converter.ConverterFactory;
import com.wmdigit.transfer.domain.CommandHeader;
import com.wmdigit.transfer.domain.DataBean;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class DataPackUtils {
    public static final int DECOMPRESS_BUF_SIZE = 1500;
    public static final int MAX_PAGE_NUM = 8;
    public static final int REC_BUF_SIZE = 1600;
    private static DataPackUtils instance;
    private CommandHeaderConverter commandHeaderConverter = new CommandHeaderConverter();
    private FixedWeightUsageEnum fixedWeightUse = FixedWeightUsageEnum.FIXED_WEIGHT;

    private DataPackUtils() {
    }

    public static synchronized DataPackUtils getInstance() {
        DataPackUtils dataPackUtils;
        synchronized (DataPackUtils.class) {
            if (instance == null) {
                instance = new DataPackUtils();
            }
            dataPackUtils = instance;
        }
        return dataPackUtils;
    }

    public DataPackWrapper parse(byte[] bArr, int i) throws IOException {
        int i2 = i - 2;
        int calcCrc16 = BeanConverter.CC.calcCrc16(bArr, 1, i2);
        byte b = (byte) ((calcCrc16 >> 8) & 255);
        byte b2 = (byte) (calcCrc16 & 255);
        if (i > 2) {
            if (bArr[i2] != b || bArr[i - 1] != b2) {
                return new DataPackWrapper(false);
            }
        }else {
            return new DataPackWrapper(false);
        }
        ByteBuffer allocate = ByteBuffer.allocate(i);
        allocate.put(bArr, 0, i);
        allocate.order(ByteOrder.LITTLE_ENDIAN).flip();
        CommandHeader bean = this.commandHeaderConverter.toBean(allocate);
        if (bean.isCompressed()) {
            bean.getPageNum();
            bean = this.commandHeaderConverter.toBean(allocate);
        }
        DataPackWrapper dataPackWrapper = new DataPackWrapper(allocate, bean);
        if (dataPackWrapper.isWrite() || dataPackWrapper.isDelete()) {
            BeanConverter create = ConverterFactory.create(bean.getScaleType(), bean.getDataType());
            if (create != null) {
                int i3 = 0;
                while (i3 < bean.getPageNum()) {
                    try {
                        allocate.position((bean.getPageLength() * i3) + 15);
                        if (dataPackWrapper.isWrite()) {
                            dataPackWrapper.addBean((DataBean) create.toBean(allocate));
                        } else {
                            dataPackWrapper.addBean((DataBean) create.toDeleteBean(allocate));
                        }
                        i3++;
                    } catch (UnsupportedEncodingException e) {
                        dataPackWrapper.setResultOk(false);
                        throw new RuntimeException(e);
                    }
                }
            }
        }
        return dataPackWrapper;
    }

    public void setOut(DataPackWrapper dataPackWrapper) {
        ScaleTypeEnum scaleType = dataPackWrapper.getInHeader().getScaleType();
        if (scaleType == null) {
            dataPackWrapper.setOutBytes(new byte[]{6});
            return;
        }
        if (ScaleTypeEnum.RL00 != scaleType || dataPackWrapper.isRead()) {
            int capacity = dataPackWrapper.getInBuffer().capacity();
            Log.d("RL00", capacity + "");
            ByteBuffer order = ByteBuffer.allocate(capacity + 1).order(ByteOrder.LITTLE_ENDIAN);
            if (!dataPackWrapper.isResultOk()) {
                int control = (dataPackWrapper.getInHeader().getControl() & ControlResponseEnum.MASK_SCALE_TYPE_CTRL.value()) | ControlResponseEnum.GENERAL_ERROR.value();
                byte b = (byte) ((control >> 8) & 255);
                byte b2 = (byte) (control & 255);
                ByteBuffer inBuffer = dataPackWrapper.getInBuffer();
                inBuffer.position(10);
                inBuffer.put(b2).put(b);
            }
            order.put((byte) 102).put(dataPackWrapper.getInBuffer().array());

            dataPackWrapper.setOutBytes(order.array());
            return;
        }
//        Log.d("RL00",   "ceshi");
        dataPackWrapper.setOutBytes(new byte[]{6});
    }

    public FixedWeightUsageEnum getFixedWeightUse() {
        return this.fixedWeightUse;
    }

    public void setFixedWeightUse(FixedWeightUsageEnum fixedWeightUsageEnum) {
        this.fixedWeightUse = fixedWeightUsageEnum;
    }

}
